<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author PrestaShop SA <contact@prestashop.com>
 * @copyright  2007-2018 PrestaShop SA
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * Class AdminKiwizController
 */
class AdminKiwizController extends ModuleAdminController
{
    /**
     * @var
     */
    public $errors;
    /**
     * @var
     */
    public $action;

    /**
     *
     */
    public function postProcess()
    {
        parent::postProcess();
    }

    /**
     *
     */
    public function initProcess()
    {
        parent::initProcess();
        $this->checkCacheFolder();

        if ($action = Tools::getValue('submitAction')) {
            $this->action = $action;
        } else {
            $this->errors[] = Tools::displayError('You do not have permission to view this.');
        }
    }

    /**
     *
     */
    public function checkCacheFolder()
    {
        if (!is_dir(_PS_CACHE_DIR_ . 'tcpdf/')) {
            mkdir(_PS_CACHE_DIR_ . 'tcpdf/');
        }
    }

    /**
     * Update invoice from api
     */
    public function processUpdateInvoiceCertification()
    {
        if (Tools::isSubmit('document_id')) {
            $invoiceId = Tools::getValue('document_id');

            $invoice = new OrderInvoice($invoiceId);
            $error = $invoice->update();
            if (is_object($error)) {
                $this->errors[] = Tools::displayError($error->getCodeError() . ' ' . $error->getMessage());
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminOrders'));
            }
        } else {
            $this->errors[] = Tools::displayError('The document id -- is missing.');
        }
    }

    /**
     * Update slip from api
     */
    public function processUpdateOrderSlipCertification()
    {
        if (Tools::isSubmit('document_id')) {
            $slipId = Tools::getValue('document_id');

            $slip = new OrderSlip($slipId);
            $error = $slip->update();
            if (is_object($error)) {
                $this->errors[] = Tools::displayError($error->getCodeError() . ' ' . $error->getMessage());
            } else {
                Tools::redirectAdmin($this->context->link->getAdminLink('AdminSlip'));
            }
        } else {
            $this->errors[] = Tools::displayError('The document id -- is missing.');
        }
    }

    /**
     * Get invoice form api
     */
    public function processGenerateInvoicePdf()
    {
        if (Tools::isSubmit('block_hash')) {
            $blockHash = Tools::getValue('block_hash');
            $documentId = Tools::getValue('document_id');

            $document = new OrderInvoice((int)$documentId);
            $fileName = str_replace(
                "#",
                "",
                $document->getInvoiceNumberFormatted($this->context->language->id)
            );

            $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi();
            try {
                $pdf = $kiwizApi->get(
                    'invoice',
                    array(
                        'name' => 'block_hash',
                        'value' => $blockHash
                    )
                );
                if (is_object($pdf)) {
                    $apiError = new ApiError();
                    $apiError->setIdOrderFile($documentId);
                    $apiError->setFileType('invoice');
                    $apiError->setCodeError($pdf->getCode());
                    $apiError->setMessage($pdf->getMessage());
                    $apiError->save();
                    $this->errors[] = Tools::displayError($pdf->getCode() . ' ' . $pdf->getMessage());
                } else {
                    $path = _PS_ROOT_DIR_ . '/upload/' . $fileName . '.pdf';

                    $fp = fopen($path, 'w');
                    fwrite($fp, $pdf);
                    fclose($fp);

                    $url = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . '/upload/' . $fileName;
                    Tools::redirect($url);
                }
            } catch (Exception $e) {
                $this->errors[] = Tools::displayError('404 Unknowned Error Call you Administrator');
            }
        } else {
            $this->errors[] =
                Tools::displayError('The order block_hash -- or the invoice order block_hash -- is missing.');
        }
    }

    /**
     * Get slip from api
     */
    public function processGenerateOrderSlipPdf()
    {
        if (Tools::isSubmit('block_hash')) {
            $blockHash = Tools::getValue('block_hash');
            $documentId = Tools::getValue('document_id');
            $fileName = Configuration::get('PS_CREDIT_SLIP_PREFIX') . '0000' . $documentId . '.pdf';

            $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi(Context::getContext()->smarty);
            try {
                $pdf = $kiwizApi->get(
                    'creditmemo',
                    array(
                        'name' => 'block_hash',
                        'value' => $blockHash
                    )
                );

                if (is_object($pdf)) {
                    $apiError = new ApiError();
                    $apiError->setIdOrderFile($documentId);
                    $apiError->setFileType('slip');
                    $apiError->setCodeError($pdf->getCode());
                    $apiError->setMessage($pdf->getMessage());
                    $apiError->save();
                    $this->errors[] = Tools::displayError($pdf->getCode() . ' ' . $pdf->getMessage());
                } else {
                    $path = _PS_ROOT_DIR_ . '/upload/' . $fileName . '.pdf';

                    $fp = fopen($path, 'w');
                    fwrite($fp, $pdf);
                    fclose($fp);

                    $url = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . '/upload/' . $fileName;
                    Tools::redirect($url);
                }
            } catch (Exception $e) {
                $this->errors[] = Tools::displayError($pdf->getCode() . ' ' . $pdf->getMessage());

                return false;
            }
        } else {
            $this->errors[] = Tools::displayError('The document id -- is missing.');
        }
    }

    /**
     * Get multiple invoices from api
     */
    public function processGenerateInvoicesPDF()
    {
        $dateFrom = Tools::getValue('date_from');
        $dateTo = Tools::getValue('date_to');

        $tableOrder = _DB_PREFIX_ . 'orders';
        $tableInvoice = _DB_PREFIX_ . "order_invoice";
        $tableKiwizData = _DB_PREFIX_ . "kiwiz_file_data";
        $sql = "SELECT * FROM  $tableInvoice";
        $sql .= " LEFT JOIN $tableKiwizData ON $tableInvoice.id_order_invoice = $tableKiwizData.id_order_file";
        $sql .= " LEFT JOIN $tableOrder ON $tableInvoice.id_order = $tableOrder.id_order";
        $sql .= " WHERE $tableKiwizData.file_type = 'invoice'";
//        $sql .= " AND $tableKiwizData.status = '1'";
        $sql .= " AND cast($tableInvoice.date_add as date) BETWEEN '$dateFrom' AND '$dateTo'";
        $sql .= " AND $tableOrder.id_shop = " . (int)Context::getContext()->shop->id;
        $results = Db::getInstance()->executeS($sql);

        $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi();
        $entity = 'invoice';
        $binaryContents = array();
        foreach ($results as $result) {
            if ($result['status'] == '1') {
                try {
                    $binaryContents[] = $kiwizApi->get(
                        $entity,
                        array(
                            'name' => 'block_hash',
                            'value' => $result['block_hash']
                        )
                    );
                } catch (Exception $e) {
                    $this->errors[] =
                        Tools::displayError('The order block_hash -- or the invoice order block_hash -- is missing.');
                }
            } else {
                $binaryContents[] =
                    $kiwizApi->generatePdf($entity, new OrderInvoice((int)$result["id_order_invoice"]), true);
            }
        }
        $fileName = 'invoice_' . $dateFrom . '-' . $dateTo . '.pdf';
        if (!empty($binaryContents)) {
            if (count($binaryContents) == 1) {
                $path = _PS_ROOT_DIR_ . '/upload/' . $fileName;

                $fp = fopen($path, 'w');
                fwrite($fp, $binaryContents[0]);
                fclose($fp);

                $url = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . '/upload/' . $fileName;
                Tools::redirect($url);
            }

            $pdf = new PDFMerger();

            $filename = 1;
            foreach ($binaryContents as $binaryContent) {
                file_put_contents(_PS_ROOT_DIR_ . '/upload/' . $filename . '.pdf', $binaryContent);
                $path = _PS_ROOT_DIR_ . '/upload/' . $filename . '.pdf';
                try {
                    $pdf->addPDF($path);
                } catch (Exception $e) {
                    $this->vd($e->getMessage());
                    die;
                }
                $filename++;
            }
//            $pdf->merge('browser', $fileName, 'P');
            $pdf->merge('browser', $fileName, true);
        } else {
            $this->errors[] = Tools::displayError('no invoices');
        }
    }

    /**
     * @param $var
     */
    public function vd($var)
    {
        echo '<pre style="color:#fff;background: #211f1f;padding: 10px;">';
        var_dump($var);
        echo '</pre>';
    }

    /**
     * Get multiple slips from api
     */
    public function processGenerateOrderSlipsPDF()
    {
        $dateFrom = Tools::getValue('date_from');
        $dateTo = Tools::getValue('date_to');

        $tableOrder = _DB_PREFIX_ . 'orders';
        $tableSlip = _DB_PREFIX_ . "order_slip";
        $tableKiwizData = _DB_PREFIX_ . "kiwiz_file_data";
        $sql = "SELECT * FROM  $tableSlip";
        $sql .= " LEFT JOIN $tableKiwizData ON $tableSlip.id_order_slip = $tableKiwizData.id_order_file";
        $sql .= " LEFT JOIN $tableOrder ON $tableSlip.id_order = $tableOrder.id_order";
        $sql .= " WHERE $tableKiwizData.file_type = 'slip' ";
//        $sql .= " AND $tableKiwizData.status = '1' ";
        $sql .= " AND cast($tableSlip.date_add as date) BETWEEN '$dateFrom' AND '$dateTo'";
        $sql .= " AND $tableOrder.id_shop = " . (int)Context::getContext()->shop->id;
        $results = Db::getInstance()->executeS($sql);

        $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi();
        $binaryContents = array();
        $entity = 'creditmemo';
        foreach ($results as $result) {
            if ($result['status'] == '1') {
                try {
                    $binaryContents[] = $kiwizApi->get(
                        $entity,
                        array(
                            'name' => 'block_hash',
                            'value' => $result['block_hash']
                        )
                    );
                } catch (Exception $e) {
                    $this->errors[] = Tools::displayError('No slips');
                }
            } else {
                $binaryContents[] =
                    $kiwizApi->generatePdf($entity, new OrderSlip((int)$result["id_order_slip"]), true);
            }
        }
        $fileName = 'slip_' . $dateFrom . '-' . $dateTo . '.pdf';
        if (!empty($binaryContents)) {
            if (count($binaryContents) == 1) {
                $path = _PS_ROOT_DIR_ . '/upload/' . $fileName;

                $fp = fopen($path, 'w');
                fwrite($fp, $binaryContents[0]);
                fclose($fp);

                $url = $_SERVER['REQUEST_SCHEME'] . '://' . $_SERVER['HTTP_HOST'] . '/upload/' . $fileName;
                Tools::redirect($url);
            }

            $pdf = new PDFMerger();
            $filename = 1;
            foreach ($binaryContents as $binaryContent) {
                file_put_contents(_PS_ROOT_DIR_ . '/upload/' . $filename, $binaryContent);
                $path = _PS_ROOT_DIR_ . '/upload/' . $filename;

                try {
                    $pdf->addPDF($path);
                } catch (Exception $e) {
                    $this->vd($e->getMessage());
                    die;
                }
                $filename++;
            }
            $pdf->merge('browser', $fileName, 'P');
        } else {
            $this->errors[] = Tools::displayError('No slips');
        }
    }

    /**
     * Update Invoices from api
     */
    public function processCronInvoices()
    {
        $tableOrder = _DB_PREFIX_ . 'orders';
        $tableInvoice = _DB_PREFIX_ . "order_invoice";
        $tableKiwizData = _DB_PREFIX_ . "kiwiz_file_data";
        $sql = "SELECT * FROM  $tableInvoice";
        $sql .= " LEFT JOIN $tableKiwizData ON $tableInvoice.id_order_invoice = $tableKiwizData.id_order_file";
        $sql .= " LEFT JOIN $tableOrder ON $tableInvoice.id_order = $tableOrder.id_order";
        $sql .= " WHERE $tableKiwizData.file_type = 'invoice'";
        $sql .= " AND $tableKiwizData.status = '0'";
        $sql .= " AND $tableOrder.id_shop = " . (int)Context::getContext()->shop->id;

        $results = Db::getInstance()->executeS($sql);

        foreach ($results as $result) {
            $invoice = new OrderInvoice($result['id_order_invoice']);
            $invoice->update();
        }
        $this->vd('cron done');

        $this->processCronSlips();
        die;
    }

    /**
     * Update Slips from api
     */
    public function processCronSlips()
    {
        $tableOrder = _DB_PREFIX_ . 'orders';
        $tableSlip = _DB_PREFIX_ . "order_slip";
        $tableKiwizData = _DB_PREFIX_ . "kiwiz_file_data";
        $sql = "SELECT * FROM  $tableSlip";
        $sql .= " LEFT JOIN $tableKiwizData ON $tableSlip.id_order_slip = $tableKiwizData.id_order_file";
        $sql .= " LEFT JOIN $tableOrder ON $tableSlip.id_order = $tableOrder.id_order";
        $sql .= " WHERE $tableKiwizData.file_type = 'slip'";
        $sql .= " AND $tableKiwizData.status = '0'";
        $sql .= " AND $tableOrder.id_shop = " . (int)Context::getContext()->shop->id;

        $results = Db::getInstance()->executeS($sql);

        foreach ($results as $result) {
            $slip = new OrderSlip($result['id_order_slip']);
            $slip->update();
        }
        $this->vd('slips done');
    }
}
