<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once(_PS_MODULE_DIR_ . 'kiwizinvoicehandler/vendor/autoload.php');
require_once _PS_MODULE_DIR_ . 'kiwizinvoicehandler/classes/FileData.php';
require_once _PS_MODULE_DIR_ . 'kiwizinvoicehandler/classes/ApiError.php';
require_once _PS_MODULE_DIR_ . 'kiwizinvoicehandler/classes/ApiCredential.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

/**
 * Class KiwizInvoiceHandler
 */
class KiwizInvoiceHandler extends Module
{
    protected $config_form = false;
    private $errors;
    private $confirmation;

    /**
     * KiwizInvoiceHandler constructor.
     */
    public function __construct()
    {
        $this->name = 'kiwizinvoicehandler';
        $this->tab = 'administration';
        $this->version = '1.0.0';
        $this->author = 'Market Academy';
        $this->need_instance = 1;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Kiwiz Invoice Handler');
        $this->description =
            $this->l('Kiwiz est une solution de certification des transactions, totalement autonome...');

        $this->confirmUninstall =
            $this->l('Do you really want to delete this module, you will loose data from your database');

        $this->ps_versions_compliancy = array('min' => '1.6', 'max' => _PS_VERSION_);
    }

    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        Configuration::updateValue('KIWIZINVOICEHANDLER_API_USERNAME', null);
        Configuration::updateValue('KIWIZINVOICEHANDLER_API_PASSWORD', null);
        Configuration::updateValue('KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID', null);
        Configuration::updateValue('KIWIZINVOICEHANDLER_API_TOKEN', null);
        Configuration::updateValue('KIWIZINVOICEHANDLER_API_NOTIFICATION_EMAILS', null);
        Configuration::updateValue('KIWIZINVOICEHANDLER_LIVE_MODE', false);

        include(dirname(__FILE__) . '/sql/install.php');

//        $this->setupDatabase();

        $tab = new Tab();
        $tab->active = 1;
        $languages = Language::getLanguages(false);

        $tab->name = array();
        if (is_array($languages)) {
            foreach ($languages as $language) {
                $tab->name[$language['id_lang']] = 'Admin Kiwiz';
            }
        }

        $tab->class_name = 'AdminKiwiz';
        $tab->module = $this->name;
        $tab->id_parent = -1;
        $tab->save();

        $secondTab = new Tab();
        $secondTab->active = 1;
        $languages = Language::getLanguages(false);

        $secondTab->name = array();
        if (is_array($languages)) {
            foreach ($languages as $language) {
                $secondTab->name[$language['id_lang']] = 'Kiwiz Error';
            }
        }

        $secondTab->class_name = 'AdminKiwizError';
        $secondTab->module = $this->name;
        $secondTab->id_parent = 18;
        $secondTab->save();

        $meta = new Meta();
        $meta->page = 'module-kiwizinvoicehandler-front';
        $meta->configurable = 1;
        $meta->title = 'Get Kiwiz Invoice';
        $meta->url_rewrite = 'kiwizinvoice';
        $meta->description = 'Generate Kiwiz invoice file';
        $meta->save();

        $this->moveTpls();

        return
            parent::install() &&
            $this->registerHook('header') &&
            $this->registerHook('backOfficeHeader') &&
            $this->registerHook('moduleRoutes');
//            && $this->registerHook('actionCronJob');
    }

    public function moveTpls()
    {
        $adminFolder = _PS_MODULE_DIR_ . 'kiwizinvoicehandler/views/templates/admin';
        $adminOverride = _PS_ROOT_DIR_ . '/override/controllers/admin/templates';
        $frontFolder = _PS_MODULE_DIR_ . 'kiwizinvoicehandler/views/templates/front';
        $frontOverride = _PS_ROOT_DIR_ . '/override/controllers/front/templates';

//        $adminControllerFolder = _PS_MODULE_DIR_ .
//            'kiwizinvoicehandler/override/controllers/admin/templates/';
//        $frontControllerFolder = _PS_MODULE_DIR_ .
//            'kiwizinvoicehandler/override/controllers/front/templates/';


        if (!file_exists($adminFolder)) {
            $this->vd('module admin folder does not exist');
        }
        if (!file_exists($frontFolder)) {
            $this->vd('module admin folder does not exist');
        }

        if (!file_exists($adminOverride)) {
            $this->vd($adminOverride);
            $this->vd('override admin folder does not exist');
        }
        if (!file_exists($frontOverride)) {
            $this->vd($frontOverride);
            $this->vd('override front folder does not exist');
            mkdir($frontOverride);
        }

        $this->recurseCopy($adminFolder, $adminOverride);
        $this->recurseCopy($frontFolder, $frontOverride);

//        copy($adminControllerFolder . 'AdminInvoicesController.php', $adminOverride);
//        copy($frontControllerFolder . 'AdminSlipController.php', $frontOverride);
    }

    public function vd($var)
    {
        echo '<pre style="color:#fff;background: #211f1f;padding: 10px;">';
        var_dump($var);
        echo '</pre>';
    }

    public function recurseCopy($src, $dst)
    {
        $dir = opendir($src);
        @mkdir($dst);
        while (false !== ($file = readdir($dir))) {
            if (($file != '.') && ($file != '..')) {
                if (is_dir($src . '/' . $file)) {
                    $this->recurseCopy($src . '/' . $file, $dst . '/' . $file);
                } else {
                    copy($src . '/' . $file, $dst . '/' . $file);
                }
            }
        }
        closedir($dir);
    }

    /**
     * @return mixed
     */
    public function uninstall()
    {
        Configuration::deleteByName('KIWIZINVOICEHANDLER_API_USERNAME');
        Configuration::deleteByName('KIWIZINVOICEHANDLER_API_PASSWORD');
        Configuration::deleteByName('KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID');
        Configuration::deleteByName('KIWIZINVOICEHANDLER_API_NOTIFICATION_EMAILS');
        Configuration::deleteByName('KIWIZINVOICEHANDLER_LIVE_MODE');

        $tabIds = array();
        $tabIds[] = (int)Tab::getIdFromClassName('AdminKiwiz');
        $tabIds[] = (int)Tab::getIdFromClassName('AdminKiwizError');
        foreach ($tabIds as $id_tab) {
            if ($id_tab) {
                $tab = new Tab($id_tab);
                $tab->delete();
            }
        }
        include(dirname(__FILE__) . '/sql/uninstall.php');

        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        $kiwiApiHandler = new KiwizInvoiceHandler\Service\KiwizApi(Context::getContext()->smarty);
        $token = $kiwiApiHandler->getApiCredential()->token;

        if ($token) {
            $quota = $kiwiApiHandler->getQuota();
//            $this->vd($quota);die;
            if (get_class($quota) == 'stdClass') {
                $this->context->smarty->assign('quota', $quota);
            } else {
//                $apiError = new ApiError();
//                $apiError->setIdOrderFile('0');
//                $apiError->setFileType('--');
//                $apiError->setCodeError($quota->getCode());
//                $apiError->setMessage($quota->getMessage());
//                $apiError->save();

                $this->context->smarty->assign('warning', $quota->getCode() . ':' . $quota->getMessage());
            }
        } else {
            $this->context->smarty->assign('warning', 'No token you need to login');
        }

        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitKiwizInvoiceHandlerModule')) == true) {
            $this->postProcess();
        }

        $this->context->smarty->assign('module_dir', $this->_path);
        $this->context->smarty->assign('link', Context::getContext()->link);

        $output = $this->context->smarty->fetch($this->local_path . 'views/templates/admin/configure.tpl');

        return $output . $this->renderForm();
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $checkPost = $this->checkPost($_POST);
        if ($checkPost) {
            if ($this->updateDatabase($_POST)) {
                $form_values = $this->getConfigFormValues();

                foreach (array_keys($form_values) as $key) {
                    Configuration::updateValue($key, Tools::getValue($key));
                }

                $this->context->smarty->clearAssign('warning');
                $this->context->smarty->clearAssign('error');
                $this->context->smarty->assign('success', $this->l('Credentials updated'));
                Configuration::updateValue('KIWIZINVOICEHANDLER_LIVE_MODE', true);
                $this->setupDatabase();
            }
        }
    }

    /**
     * @param $post
     * @return bool
     */
    public function checkPost($post)
    {
        if (!array_key_exists('KIWIZINVOICEHANDLER_API_USERNAME', $post)
            || !array_key_exists('KIWIZINVOICEHANDLER_API_PASSWORD', $post)
            || !array_key_exists('KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID', $post)
        ) {
            $this->context->smarty->assign('error', $this->l('key missing in POST'));

            return false;
        }

        if (empty($post['KIWIZINVOICEHANDLER_API_USERNAME'])) {
            $this->context->smarty->assign('error', $this->l('invalid username'));

            return false;
        }

        if (empty($post['KIWIZINVOICEHANDLER_API_PASSWORD'])) {
            $this->context->smarty->assign('error', $this->l('invalid password'));

            return false;
        }

        if (empty($post['KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID'])) {
            $this->context->smarty->assign('error', $this->l('invalid subscription id'));

            return false;
        }

        return true;
    }

    /**
     * @param $post
     * @return bool
     * @throws PrestaShopException
     */
    public function updateDatabase($post)
    {
        $email = $post['KIWIZINVOICEHANDLER_API_USERNAME'];
        $password = $post['KIWIZINVOICEHANDLER_API_PASSWORD'];
        $subscriptionId = $post['KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID'];
        $notificationEmails = $post['KIWIZINVOICEHANDLER_API_NOTIFICATION_EMAILS'];

        $kiwiApiHandler = new KiwizInvoiceHandler\Service\KiwizApi(Context::getContext()->smarty);
        $token = $kiwiApiHandler->generateToken($email, $password, $subscriptionId, $notificationEmails);
        if (is_object($token)) {
            $this->context->smarty->clearAssign('warning');
            $this->context->smarty->clearAssign('error');

            $this->context->smarty->assign('error', $token->getMessage());

            return false;
        }

        return true;
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {

        return array(
            'KIWIZINVOICEHANDLER_API_USERNAME' =>
                Configuration::get('KIWIZINVOICEHANDLER_API_USERNAME', null),
            'KIWIZINVOICEHANDLER_API_PASSWORD' =>
                Configuration::get('KIWIZINVOICEHANDLER_API_PASSWORD', null),
            'KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID' =>
                Configuration::get('KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID', null),
            'KIWIZINVOICEHANDLER_API_TOKEN' =>
                Configuration::get('KIWIZINVOICEHANDLER_API_TOKEN', null),
            'KIWIZINVOICEHANDLER_API_NOTIFICATION_EMAILS' =>
                Configuration::get('KIWIZINVOICEHANDLER_API_NOTIFICATION_EMAILS', null),
        );
    }

    public function setupDatabase()
    {
        $orderTable = _DB_PREFIX_ . 'orders';
        $invoiceTable = _DB_PREFIX_ . "order_invoice";
        $sql = "SELECT id_order_invoice, id_shop FROM  $invoiceTable ";
        $sql .= " LEFT JOIN $orderTable ON $invoiceTable.id_order = $orderTable.id_order";
        $sql .= " WHERE id_shop = " . (int)Context::getContext()->shop->id;
        $invoices = Db::getInstance()->executeS($sql);

        foreach ($invoices as $invoice) {
            try {
                $fileData = new FileData();
                $fileData->setIdOrderFile($invoice['id_order_invoice']);
                $fileData->setFileType('invoice');
                $fileData->setStatus(-1);
                $fileData->save();
            } catch (PrestaShopExceptionCore $e) {
                $this->context->smarty->assign('success', $e->displayMessage());
            }
        }

        $slipTable = _DB_PREFIX_ . "order_slip";
        $sql = "SELECT id_order_slip, id_shop FROM  $slipTable ";
        $sql .= " LEFT JOIN $orderTable ON $slipTable.id_order = $orderTable.id_order";
        $sql .= " WHERE id_shop = " . (int)Context::getContext()->shop->id;
        $slips = Db::getInstance()->executeS($sql);

        foreach ($slips as $slip) {
            try {
                $fileData = new FileData();
                $fileData->setIdOrderFile($slip['id_order_slip']);
                $fileData->setFileType('slip');
                $fileData->setStatus(-1);
                $fileData->save();
            } catch (PrestaShopExceptionCore $e) {
                $this->context->smarty->assign('success', $e->displayMessage());
            }
        }
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();

        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitKiwizInvoiceHandlerModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
            . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');

        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($this->getConfigForm()));
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        return array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('API Credentials'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-envelope"></i>',
                        'desc' => $this->l('Enter a valid email address'),
                        'name' => 'KIWIZINVOICEHANDLER_API_USERNAME',
                        'label' => $this->l('Email'),
                    ),
                    array(
                        'col' => 3,
                        'prefix' => '<i class="icon icon-key"></i>',
                        'type' => 'password',
                        'name' => 'KIWIZINVOICEHANDLER_API_PASSWORD',
                        'label' => $this->l('Password'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'text',
                        'prefix' => '<i class="icon icon-key"></i>',
                        'desc' => $this->l('Enter you subscrption_id'),
                        'name' => 'KIWIZINVOICEHANDLER_API_SUBSCRIPTION_ID',
                        'label' => $this->l('Subscription_id'),
                    ),
                    array(
                        'col' => 3,
                        'type' => 'textarea',
                        'prefix' => '<i class="icon icon-envelope-alt"></i>',
                        'desc' => $this->l('enter emails with ; between emails'),
                        'name' => 'KIWIZINVOICEHANDLER_API_NOTIFICATION_EMAILS',
                        'label' => $this->l('notification_emails'),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );
    }

    public function getNotificationEmail()
    {
        $sql = "SELECT notification_emails FROM " . _DB_PREFIX_ . "kiwiz_invoice_handler";
        $sql .= " WHERE id_shop = " . (int)Context::getContext()->shop->id;
        $result = Db::getInstance()->getRow($sql);
        if (!$result) {
            return false;
        }

        return $result['notification_emails'];
    }

    public function getInvoiceId($idOrder)
    {
        $order = new OrderCore($idOrder);
        $invoices = $order->getInvoicesCollection();

        $dataInvoice = null;
        foreach ($invoices as $invoice) {
            $dataInvoice = $invoice;
        }

        return $dataInvoice->id;
    }

    public function getOrderStatus($documents)
    {
        if (empty($documents)) {
            return false;
        }

        foreach ($documents as $document) {
//            $class = get_class($document);
            $fileData = $document->getKiwizData();

            if (!$fileData) {
                return false;
            }

            if ($fileData->getStatus() != "1") {
                return false;
            }
        }

        return true;
    }

    public function hookModuleRoutes($params)
    {
        $my_link = array(
            'kiwizinvoice' => array(
                'controller' => 'front',
                'rule' => '',
                'params' => array()
            )
        );

        return $my_link;
    }

    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS($this->_path . 'views/js/back.js');
            $this->context->controller->addCSS($this->_path . 'views/css/back.css');
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS($this->_path . '/views/css/front.css');
    }
}
