<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

require_once _PS_MODULE_DIR_ . 'kiwizinvoicehandler/classes/FileData.php';

class OrderInvoice extends OrderInvoiceCore
{
    private $file_type = 'invoice';
    private $productList = array();

    public function __construct($id = null, $id_lang = null, $id_shop = null)
    {
        parent::__construct($id, $id_lang, $id_shop);
    }


    public function getProductList()
    {
        return $this->productList;
    }

    public function add($auto_date = true, $null_values = false)
    {
        $this->productList = $this->getProducts();

        parent::add($auto_date, $null_values);

        if ((int)$this->number == 0) {
            $this->number = (int)Order::getLastInvoiceNumber() + 1;
        }

        if (Configuration::get('KIWIZINVOICEHANDLER_LIVE_MODE')) {
            $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi(Context::getContext()->smarty);
            $apiResponse = $kiwizApi->save($this->file_type, $this);
            $fileData = new FileData();
            $fileData->setIdOrderFile($this->id);
            $fileData->setFileType($this->file_type);
            if (get_class($apiResponse) == "stdClass") {
                try {
                    $fileData->setFileHash($apiResponse->file_hash);
                    $fileData->setBlockHash($apiResponse->block_hash);
                    $fileData->setStatus(1);
                    $fileData->add();
                } catch (PrestaShopException $e) {
                    $this->vd($e->displayMessage());
                    die;
                }
            } else {
                $fileData->setStatus(0);
                $fileData->add();
                $apiError = new ApiError();
                $apiError->setIdOrderFile($this->id);
                $apiError->setFileType($this->file_type);
                $apiError->setCodeError($apiResponse->getCode());
                $apiError->setMessage($apiResponse->getMessage());
                $apiError->save();

                $notificationEmailsString = $kiwizApi->getApiCredential()->notification_emails;
                $this->sendEmail($notificationEmailsString, $apiResponse);

                return $apiError;
            }
        }

        return true;
    }

    public function vd($var)
    {
        echo '<pre style="color:#fff;background: #211f1f;padding: 10px;">';
        var_dump($var);
        echo '</pre>';
    }

    public function sendEmail($notificationEmailsString, $apiResponse)
    {
        $notificationEmails = explode(';', $notificationEmailsString);

        $mail = new MailCore();
        $mail::Send(
            (int)Configuration::get('PS_LANG_DEFAULT'),
            'error_call_api',
            Mail::l('Error Call API'),
            array(
                '{idOrderFile}' => $this->id,
                '{fileType}' => $this->file_type,
                '{code}' => $apiResponse->getCode(),
                '{message}' => $apiResponse->getMessage()
            ),
            $notificationEmails,
            null,
            null,
            null,
            null,
            null,
            _PS_MODULE_DIR_ . 'kiwizinvoicehandler/mails/'
        );
    }

    public function update($null_values = false)
    {
        if ((int)$this->number == 0) {
            $this->number = Order::getLastInvoiceNumber();

            $apiError = new ApiError();
            $apiError->setIdOrderFile($this->id);
            $apiError->setFileType($this->file_type);
            $apiError->setCodeError('500');
            $apiError->setMessage('Invoice number 0 new number : ' . $this->number);
            $apiError->save();
        }
        parent::update($null_values);


        if (Configuration::get('KIWIZINVOICEHANDLER_LIVE_MODE')) {
            $fileData = $this->getKiwizData();

            if ($fileData) {
                if ($fileData->getStatus() == 0) {
                    $kiwizApi = new KiwizInvoiceHandler\Service\KiwizApi(Context::getContext()->smarty);
                    $apiResponse = $kiwizApi->save($this->file_type, $this);

                    $fileData->setIdOrderFile($this->id);
                    $fileData->setFileType($this->file_type);
                    if (get_class($apiResponse) == "stdClass") {
                        try {
                            $fileData->setFileHash($apiResponse->file_hash);
                            $fileData->setBlockHash($apiResponse->block_hash);
                            $fileData->setStatus(1);
                            $fileData->save();
                        } catch (Exception $e) {
                            $this->vd($e->displayMessage());
                            die;
                        }
                    } else {
                        $fileData->setStatus(0);
                        $fileData->save();
                        $apiError = new ApiError();
                        $apiError->setIdOrderFile($this->id);
                        $apiError->setFileType($this->file_type);
                        $apiError->setCodeError($apiResponse->getCode());
                        $apiError->setMessage($apiResponse->getMessage());
                        $apiError->save();
                        $notificationEmailsString = $kiwizApi->getApiCredential()->notification_emails;
                        $this->sendEmail($notificationEmailsString, $apiResponse);
                        return $apiError;
                    }
                }
            } else {
                $fileData = new FileData();
                $fileData->setStatus(0);
                $fileData->setFileType($this->file_type);
                $fileData->save();
            }
        }

        return true;
    }

    public function getKiwizData()
    {
        $sql = "SELECT *
			FROM `" . _DB_PREFIX_ . "kiwiz_file_data`
			WHERE id_order_file = " . (int)$this->id .
            " AND file_type = " . "'invoice'";
        $result = Db::getInstance()->getRow($sql);
        if ($result) {
            $fileData = new FileData();
            $fileData->setId($result['id']);
            $fileData->setIdOrderFile($result['id_order_file']);
            $fileData->setFileHash($result['file_hash']);
            $fileData->setBlockHash($result['block_hash']);
            $fileData->setFileType($result['file_type']);
            $fileData->setStatus($result['status']);

            return $fileData;
        }

        return $result;
    }
}
