<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2018 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

class AdminInvoicesController extends AdminInvoicesControllerCore
{
    public function __construct()
    {
        parent::__construct();
        $this->bootstrap = true;
        $this->table = 'order_invoice';
        $this->className = 'OrderInvoice';
        $this->_select = ' o.`id_shop`';
        $this->_join .= ' LEFT JOIN ' . _DB_PREFIX_ . 'orders o ON (o.`id_order` = a.`id_order`)';
        $this->_join .= ' LEFT JOIN '
            . _DB_PREFIX_ . 'kiwiz_file_data kfd ON (kfd.`id_order_file` = a.`id_order_invoice`)'
            . ' AND  (kfd.`file_type` = "invoice")';
        $this->_group = ' GROUP BY a.`id_order_invoice`';
        $this->_orderBy = 'id_order_invoice';
        $this->fields_list = array(
            'id_order_invoice' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs'
            ),
            'id_order' => array(
                'title' => $this->l('Order ID'),
                'align' => 'left',
                'class' => 'fixed-width-md',
                'havingFilter' => true
            ),
            'date_add' => array(
                'title' => $this->l('Date of invoice'),
                'type' => 'date',
                'align' => 'right',
                'filter_key' => 'a!date_add'
            ),
            'status' => array(
                'title' => $this->l('Status'),
                'align' => 'center',
                'filter_key' => 'kfd!status',
                'callback' => 'printStatus'
            ),
            'block_hash' => array(
                'title' => $this->l('Action'),
                'align' => 'center',
                'callback' => 'printActionKiwiz',
                'orderby' => false,
                'search' => false,
            ),
        );
        $this->_select .= ', kfd.status AS status,';
        $this->_select .= ' kfd.file_hash AS file_hash,';
        $this->_select .= ' kfd.block_hash AS block_hash';
        $this->optionTitle = $this->l('Invoice');

        $this->_where = Shop::addSqlRestriction(false, 'o');
    }

    public function printStatus($value)
    {
        if ($value == 1) {
            $result = $this->l('Sent');
        } elseif ($value == 0) {
            $result = $this->l('To Send');
        } elseif ($value == -1) {
            $result = $this->l('Do not send');
        }
        return $result;
    }

    public function printActionKiwiz($value, $tr)
    {
        $this->context->smarty->assign(array(
            'block_hash' => $value,
            'tr' => $tr
        ));

        return $this->createTemplate('_get_from_kiwiz.tpl')->fetch();
    }

    public function initContent()
    {
        $this->content .= $this->renderList();
        parent::initContent();
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitAddinvoice_date')) {
            if (!Validate::isDate(Tools::getValue('date_from'))) {
                $this->errors[] = $this->l('Invalid "From" date');
            }

            if (!Validate::isDate(Tools::getValue('date_to'))) {
                $this->errors[] = $this->l('Invalid "To" date');
            }

            if (!count($this->errors)) {
                if (count(
                    OrderInvoice::getByDateInterval(
                        Tools::getValue('date_from'),
                        Tools::getValue('date_to')
                    )
                )
                ) {
                    if (Configuration::get('KIWIZINVOICEHANDLER_LIVE_MODE') == true) {
                        Tools::redirectAdmin(
                            $this->context->link->getAdminLink('AdminKiwiz')
                            . '&submitAction=generateInvoicesPDF&date_from='
                            . urlencode(Tools::getValue('date_from'))
                            . '&date_to=' . urlencode(
                                Tools::getValue('date_to')
                            )
                        );
                    } else {
                        Tools::redirectAdmin(
                            $this->context->link->getAdminLink('AdminPdf')
                            . '&submitAction=generateInvoicesPDF&date_from='
                            . urlencode(
                                Tools::getValue('date_from')
                            )
                            . '&date_to='
                            . urlencode(
                                Tools::getValue('date_to')
                            )
                        );
                    }
                }

                $this->errors[] = $this->l('No invoice has been found for this period.');
            }
        } elseif (Tools::isSubmit('submitAddinvoice_status')) {
            if (!is_array($status_array = Tools::getValue('id_order_state')) || !count($status_array)) {
                $this->errors[] = $this->l('You must select at least one order status.');
            } else {
                foreach ($status_array as $id_order_state) {
                    if (count(OrderInvoice::getByStatus((int)$id_order_state))) {
                        Tools::redirectAdmin(
                            $this->context->link->getAdminLink('AdminPdf')
                            . '&submitAction=generateInvoicesPDF2&id_order_state='
                            . implode(
                                '-',
                                $status_array
                            )
                        );
                    }
                }

                $this->errors[] = $this->l('No invoice has been found for this status.');
            }
        } else {
            parent::postProcess();
        }
    }
}
