<?php
/**
 * 2007-2018 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/osl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author PrestaShop SA <contact@prestashop.com>
 * @copyright  2007-2018 PrestaShop SA
 * @license    http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

/**
 * @property AdminSlipController $object
 */
class AdminSlipController extends AdminSlipControllerCore
{
    public function __construct()
    {
        parent::__construct();
        $this->bootstrap = true;
        $this->table = 'order_slip';
        $this->className = 'OrderSlip';
        $this->_select = ' o.`id_shop`';
        $this->_join .= ' LEFT JOIN '
            . _DB_PREFIX_ . 'kiwiz_file_data kfd ON (kfd.`id_order_file` = a.`id_order_slip`)'
            . ' AND  (kfd.`file_type` = "slip")';
        $this->_group = ' GROUP BY a.`id_order_slip`';
        $this->fields_list = array(
            'id_order_slip' => array(
                'title' => $this->l('ID'),
                'align' => 'center',
                'class' => 'fixed-width-xs'
            ),
            'id_order' => array(
                'title' => $this->l('Order ID'),
                'align' => 'left',
                'class' => 'fixed-width-md',
                'havingFilter' => true
            ),
            'date_add' => array(
                'title' => $this->l('Date issued'),
                'type' => 'date',
                'align' => 'right',
                'filter_key' => 'a!date_add'
            ),
            'status' => array(
                'title' => $this->l('Status'),
                'align' => 'center',
                'filter_key' => 'kfd!status',
                'callback' => 'printStatus'
            ),
            'block_hash' => array(
                'title' => $this->l('Action'),
                'align' => 'center',
                'callback' => 'printActionKiwiz',
                'orderby' => false,
                'search' => false,
            ),
        );
        $this->_select .= ', a.id_order_slip AS id_pdf,';
        $this->_select .= ' kfd.status AS status,';
        $this->_select .= ' kfd.file_hash AS file_hash,';
        $this->_select .= ' kfd.block_hash AS block_hash';
        $this->optionTitle = $this->l('Slip');
        $this->fields_options = array(
            'general' => array(
                'title' => $this->l('Credit slip options'),
                'fields' => array(
                    'PS_CREDIT_SLIP_PREFIX' => array(
                        'title' => $this->l('Credit slip prefix'),
                        'desc' => $this->l('Prefix used for credit slips.'),
                        'size' => 6,
                        'type' => 'textLang'
                    )
                ),
                'submit' => array('title' => $this->l('Save'))
            )
        );
        $this->_where = Shop::addSqlRestriction(false, 'o');
    }

    public function postProcess()
    {
        if (Tools::getValue('submitAddorder_slip')) {
            if (!Validate::isDate(Tools::getValue('date_from'))) {
                $this->errors[] = $this->l('Invalid "From" date');
            }
            if (!Validate::isDate(Tools::getValue('date_to'))) {
                $this->errors[] = $this->l('Invalid "To" date');
            }

            if (!count($this->errors)) {
                $order_slips =
                    OrderSlip::getSlipsIdByDate(Tools::getValue('date_from'), Tools::getValue('date_to'));
                if (count($order_slips)) {
                    if (Configuration::get('KIWIZINVOICEHANDLER_LIVE_MODE') == true) {
                        Tools::redirectAdmin(
                            $this->context->link->getAdminLink('AdminKiwiz')
                            . '&submitAction=generateOrderSlipsPDF&date_from='
                            . urlencode(Tools::getValue('date_from'))
                            . '&date_to=' . urlencode(Tools::getValue('date_to'))
                        );
                    } else {
                        Tools::redirectAdmin(
                            $this->context->link->getAdminLink('AdminPdf')
                            . '&submitAction=generateOrderSlipsPDF&date_from='
                            . urlencode(Tools::getValue('date_from'))
                            . '&date_to='
                            . urlencode(Tools::getValue('date_to'))
                        );
                    }
                }
                $this->errors[] = $this->l('No order slips were found for this period.');
            }
        } else {
            return parent::postProcess();
        }
    }

    public function printActionKiwiz($value, $tr)
    {
        $this->context->smarty->assign(array(
            'block_hash' => $value,
            'tr' => $tr
        ));

        return $this->createTemplate('_get_from_kiwiz.tpl')->fetch();
    }

    public function printStatus($value)
    {
        if ($value == '1') {
            $result = $this->l('Sent');
        } elseif ($value == '0') {
            $result = $this->l('To Send');
        } elseif ($value == '-1') {
            $result = $this->l('Do not send');
        }
        return $result;
    }
}
